@@ -15,16 +15,55 @@
     along with this program; if not, write to the Free Software
     Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-/* some basic #defines to support pre-ANSI compilers */
-
 #ifndef BASICDEFS_H
 #define BASICDEFS_H
 
+#if defined(_AIX)
+#pragma alloca
+#else
+# if !defined(alloca)           /* predefined by HP cc +Olibcalls */
+#  ifdef __GNUC__
+#   define alloca(size) __builtin_alloca(size)
+#  else
+#   if HAVE_ALLOCA_H
+#    include <alloca.h>
+#   else
+#    if defined(__hpux)
+        void *alloca ();
+#    else
+#     if !defined(__OS2__) && !defined(WIN32)
+	char *alloca ();
+#     else
+#      include <malloc.h>       /* OS/2 defines alloca in here */
+#     endif
+#    endif
+#   endif
+#  endif
+# endif
+#endif
+
+#ifdef HAVE_WCHAR_H
+# include <wchar.h>
+#endif
+#ifdef HAVE_WCTYPE_H
+# include <wctype.h>
+#endif
+
+
+#ifdef BOOTSTRAP
+# define false 0
+# define true 1
+# define bool unsigned
+# define __bool_true_false_are_defined 1
+#else
+# include <stdbool.h>
+#endif
+
 #if ENABLE_NLS
 # include <libintl.h>
 #else
 # define gettext(msgid) (msgid)
-# define ngettext(sing, plur, n) ((n) == 1 ? (sing) : (plur))
+# define ngettext(n, sing, plur) ((n) == 1 ? (sing) : (plur))
 #endif
 #define _(String) gettext(String)
 
@@ -37,26 +76,6 @@
 /* type countT is used to keep track of line numbers, etc. */
 typedef unsigned long countT;
 
-/* type flagT is used for boolean values */
-#if defined FALSE && FALSE != 0
-#error Huh, FALSE is not false?
-#endif
-#if defined TRUE && TRUE != 1
-#error Huh, TRUE is not true?
-#endif
-#undef FALSE
-#undef TRUE
-
-#if __STDC__
-/* Define an enum for debugging */
-typedef enum { FALSE, TRUE } flagT;
-#else
-typedef int flagT;
-#endif
-
-#define FALSE 0
-#define TRUE 1
-
 /* Oftentimes casts are used as an ugly hack to silence warnings
  * from the compiler.  However, sometimes those warnings really
  * do point to something worth avoiding.  I define this
