@@ -255,7 +255,7 @@ savchar(ch)
   if (prog.cur)
     {
       if (prog.cur <= prog.base || *--prog.cur != ch)
-	panic(_("Called savchar() with unexpected pushback (%x)"),
+	panic("Called savchar() with unexpected pushback (%x)",
 	      CAST(unsigned char)ch);
     }
   else
@@ -588,8 +588,8 @@ match_slash(slash, regex)
   return NULL;
 }
 
-static bool mark_subst_opts P_((struct subst *cmd));
-static bool
+static int mark_subst_opts P_((struct subst *cmd));
+static int
 mark_subst_opts(cmd)
   struct subst *cmd;
 {
@@ -1641,7 +1641,7 @@ check_final_program(program)
       else
 	{
 	  if (*go->name)
-	    panic(_("Can't find label for jump to `%s'"), go->name);
+	    panic(_("can't find label for jump to `%s'"), go->name);
 	  program->v[go->v_index].x.jump_index = program->v_length;
 	}
     }
