@@ -16,50 +16,19 @@
     along with this program; if not, write to the Free Software
     Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#if defined(_AIX)
-#pragma alloca
-#else
-# if !defined(alloca)           /* predefined by HP cc +Olibcalls */
-#  ifdef __GNUC__
-#   define alloca(size) __builtin_alloca(size)
-#  else
-#   if HAVE_ALLOCA_H
-#    include <alloca.h>
-#   else
-#    if defined(__hpux)
-        void *alloca ();
-#    else
-#     if !defined(__OS2__) && !defined(WIN32)
-	char *alloca ();
-#     else
-#      include <malloc.h>       /* OS/2 defines alloca in here */
-#     endif
-#    endif
-#   endif
-#  endif
-# endif
-#endif
+#include "basicdefs.h"
+#include "regex.h"
 
 #ifndef BOOTSTRAP
 #include <stdio.h>
 #endif
-#include "basicdefs.h"
-#include "regex.h"
+
 #include "utils.h"
 
-#ifdef HAVE_WCHAR_H
-# include <wchar.h>
-#endif
-#ifdef HAVE_WCTYPE_H
-# include <wctype.h>
-#endif
-
-
 /* Struct vector is used to describe a compiled sed program. */
 struct vector {
   struct sed_cmd *v;	/* a dynamically allocated array */
@@ -67,6 +36,16 @@ struct vector {
   size_t v_length;	/* ... number of slots in use */
 };
 
+/* This structure tracks files used by sed so that they may all be
+   closed cleanly at normal program termination.  A flag is kept that tells
+   if a missing newline was encountered, so that it is added on the
+   next line and the two lines are not concatenated.  */
+struct output {
+  char *name;
+  bool missing_newline;
+  FILE *fp;
+  struct output *link;
+};
 
 struct text_buf {
   char *text;
@@ -74,29 +53,46 @@ struct text_buf {
 };
 
 enum replacement_types {
-  repl_asis = 0,
-  repl_uppercase = 1,
-  repl_lowercase = 2,
-  repl_uppercase_first = 4,
-  repl_lowercase_first = 8,
-  repl_modifiers = repl_uppercase_first | repl_lowercase_first,
+  REPL_ASIS = 0,
+  REPL_UPPERCASE = 1,
+  REPL_LOWERCASE = 2,
+  REPL_UPPERCASE_FIRST = 4,
+  REPL_LOWERCASE_FIRST = 8,
+  REPL_MODIFIERS = REPL_UPPERCASE_FIRST | REPL_LOWERCASE_FIRST,
 
   /* These are given to aid in debugging */
-  repl_uppercase_uppercase = repl_uppercase_first | repl_uppercase,
-  repl_uppercase_lowercase = repl_uppercase_first | repl_lowercase,
-  repl_lowercase_uppercase = repl_lowercase_first | repl_uppercase,
-  repl_lowercase_lowercase = repl_lowercase_first | repl_lowercase
+  REPL_UPPERCASE_UPPERCASE = REPL_UPPERCASE_FIRST | REPL_UPPERCASE,
+  REPL_UPPERCASE_LOWERCASE = REPL_UPPERCASE_FIRST | REPL_LOWERCASE,
+  REPL_LOWERCASE_UPPERCASE = REPL_LOWERCASE_FIRST | REPL_UPPERCASE,
+  REPL_LOWERCASE_LOWERCASE = REPL_LOWERCASE_FIRST | REPL_LOWERCASE
+};
+
+enum text_types {
+  TEXT_BUFFER,
+  TEXT_REPLACEMENT,
+  TEXT_REGEX
+};
+
+enum posixicity_types {
+  POSIXLY_EXTENDED,	/* with GNU extensions */
+  POSIXLY_CORRECT,	/* with POSIX-compatible GNU extensions */
+  POSIXLY_BASIC		/* pedantically POSIX */
+};
+
+enum addr_state {
+  RANGE_INACTIVE,	/* never been active */
+  RANGE_ACTIVE,		/* between first and second address */
+  RANGE_CLOSED		/* like RANGE_INACTIVE, but range has ended once */
 };
 
 enum addr_types {
-  addr_is_null,		/* null address */
-  addr_is_regex,	/* a.addr_regex is valid */
-  addr_is_num,		/* a.addr_number is valid */
-  addr_is_num_mod,	/* a.addr_number is valid, addr_step is modulo */
-  addr_is_num2,		/* a.addr_number is valid (only valid for addr2) */
-  addr_is_step,		/* address is +N (only valid for addr2) */
-  addr_is_step_mod,	/* address is ~N (only valid for addr2) */
-  addr_is_last		/* address is $ */
+  ADDR_IS_NULL,		/* null address */
+  ADDR_IS_REGEX,	/* a.addr_regex is valid */
+  ADDR_IS_NUM,		/* a.addr_number is valid */
+  ADDR_IS_NUM_MOD,	/* a.addr_number is valid, addr_step is modulo */
+  ADDR_IS_STEP,		/* address is +N (only valid for addr2) */
+  ADDR_IS_STEP_MOD,	/* address is ~N (only valid for addr2) */
+  ADDR_IS_LAST		/* address is $ */
 };
 
 struct addr {
@@ -119,7 +115,7 @@ struct subst {
   regex_t *regx;
   struct replacement *replacement;
   countT numb;		/* if >0, only substitute for match number "numb" */
-  FILE *fp;		/* 'w' option given */
+  struct output *outf;	/* 'w' option given */
   unsigned global : 1;	/* 'g' option given */
   unsigned print : 2;	/* 'p' option given (before/after eval) */
   unsigned eval : 1;	/* 'e' option given */
@@ -143,36 +139,39 @@ struct sed_cmd {
   struct addr *a1;	/* save space: usually is NULL */
   struct addr *a2;
 
-  /* non-zero if a1 has been matched; apply command until a2 matches */
-  char a1_matched;
+  /* See description the enum, above.  */
+  enum addr_state range_state;
 
-  /* non-zero: only apply command to non-matches */
+  /* Non-zero if command is to be applied to non-matches. */
   char addr_bang;
 
-  /* the actual command */
+  /* The actual command character. */
   char cmd;
 
   /* auxiliary data for various commands */
   union {
-    /* This structure is used for a, i, and c commands */
+    /* This structure is used for a, i, and c commands. */
     struct text_buf cmd_txt;
 
-    /* This is used for the l, q and Q commands */
+    /* This is used for the l, q and Q commands. */
     int int_arg;
 
-    /* This is used for {}, b, and t commands */
+    /* This is used for the {}, b, and t commands. */
     countT jump_index;
 
-    /* This for r command */
+    /* This is used for the r command. */
     char *fname;
 
-    /* This for the hairy s command */
+    /* This is used for the hairy s command. */
     struct subst *cmd_subst;
 
-    /* This for R and w command */
+    /* This is used for the w command. */
+    struct output *outf;
+
+    /* This is used for the R command. */
     FILE *fp;
 
-    /* This for the y command */
+    /* This is used for the y command. */
     unsigned char *translate;
     char **translatemb;
   } x;
@@ -181,7 +180,7 @@ struct sed_cmd {
 
 
 void bad_prog P_((const char *why));
-size_t normalize_text P_((char *, size_t));
+size_t normalize_text P_((char *text, size_t len, enum text_types buftype));
 struct vector *compile_string P_((struct vector *, char *str, size_t len));
 struct vector *compile_file P_((struct vector *, const char *cmdfile));
 void check_final_program P_((struct vector *));
@@ -200,24 +199,55 @@ int process_files P_((struct vector *, char **argv));
 
 int main P_((int, char **));
 
-extern void fmt P_ ((char *line, char *line_end, int max_length, FILE *output_file));
+extern void fmt P_ ((const char *line, const char *line_end, int max_length, FILE *output_file));
 
 extern int extended_regexp_flags;
 
 /* If set, fflush(stdout) on every line output. */
-extern flagT unbuffered_output;
+extern bool unbuffered_output;
 
 /* If set, don't write out the line unless explicitly told to. */
-extern flagT no_default_output;
+extern bool no_default_output;
 
 /* If set, reset line counts on every new file. */
-extern flagT separate_files;
+extern bool separate_files;
 
 /* Do we need to be pedantically POSIX compliant? */
-extern flagT POSIXLY_CORRECT;
+extern enum posixicity_types posixicity;
 
 /* How long should the `l' command's output line be? */
 extern countT lcmd_out_line_len;
 
 /* How do we edit files in-place? (we don't if NULL) */
 extern char *in_place_extension;
+
+/* Should we use EREs? */
+extern bool use_extended_syntax_p;
+
+/* Declarations for multibyte character sets.  */
+extern int mb_cur_max;
+
+#ifdef HAVE_MBRTOWC
+#ifdef HAVE_BTOWC
+#define MBRTOWC(pwc, s, n, ps) \
+  (mb_cur_max == 1 ? \
+   (*(pwc) = btowc (*(unsigned char *) (s)), 1) : \
+   mbrtowc ((pwc), (s), (n), (ps)))
+#else
+#define MBRTOWC(pwc, s, n, ps) \
+  mbrtowc ((pwc), (s), (n), (ps))
+#endif
+
+#define MBRLEN(s, n, ps) \
+  (mb_cur_max == 1 ? 1 : mbrtowc (NULL, s, n, ps))
+
+#define BRLEN(ch, ps) \
+  (mb_cur_max == 1 ? 1 : brlen (ch, ps))
+
+#else
+#define MBRLEN(s, n, ps) 1
+#define BRLEN(ch, ps) 1
+#endif
+
+extern int brlen P_ ((int ch, mbstate_t *ps));
+
